@@ -1,20 +1,14 @@
 # -*- coding: iso-8859-1 -*-
 """
-    MoinMoin - Formatter Base Class
+    MoinMoin - DOM XML Formatter
 
-    @copyright: 2000-2002 by Jrgen Hermann <jh@web.de>
+    @copyright: 2000-2004 by Jrgen Hermann <jh@web.de>
     @license: GNU GPL, see COPYING for details.
 """
 
-# Imports
-import sys, cgi
 from xml.dom import minidom
 from MoinMoin.formatter.base import FormatterBase
 
-#############################################################################
-### Formatter Base
-#############################################################################
-
 #def print_dom(element, indent=''):
 #    print indent + element.tagName
 #    for child in element.get
@@ -32,12 +26,16 @@ class Formatter(FormatterBase):
     format_tags = ['b', 'em', 'highlight', 'sup', 'sub', 'code', 'u']
 
     unbreakables = ['h1', 'h2', 'h3', 'h4', 'h5', 'h6',
-                    'p', 'ol', 'ul', 'li', 'pre', 'a']
+                    'p', 'ol', 'ul', 'li', 'pre', 'a',
+                    'table', 'td', 'tr']
 
-    need_p = format_tags[:]
-    need_p.extend(['ol', 'a'])
+    need_p = [] #format_tags[:]
+    need_p.extend(['ol', 'a', 'pagelink', 'interwiki', 'macro']) #XXX add more
 
-    no_p_after = ['h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'p', 'ol', 'ul', 'pre']
+    no_p_after = ['h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'p', 'ol', 'ul', 'pre',
+                  'small', 'big', 'table', 'td', 'tr', 'dt',
+                  'codearea', 'codeline', 'codetoken',
+                  'sysmesg']
 
     close_on_open = {
         'h1' : ['p'],
@@ -49,6 +47,8 @@ class Formatter(FormatterBase):
     for i in xrange(2,7):
         close_on_open['h%i' % i] = close_on_open['h1']
 
+    close_on_open = {} # XXX
+
     close_on_close = {
         'table' : ['td', 'tr'],
         'td' : ['tr'],
@@ -56,12 +56,14 @@ class Formatter(FormatterBase):
         'ol' : ['li'],
         'ul' : ['li'],
         }
+    close_on_close = {} #XXX
 
     def __init__(self, request, **kw):
         self.request = request
         self._ = request.getText
 
         self._store_pagelinks = kw.get('store_pagelinks', 0)
+        self._highlight_re = None
         self.pagelinks = []
         self.in_p = 0
         self.in_pre = 0
@@ -74,12 +76,16 @@ class Formatter(FormatterBase):
         self.page = page
 
     def _open_tag(self, tag, **attrs):
-        """ low level function: opens tag right now """
+        """ low level function: opens tag right now
+
+        @param tag: tag name, string
+        @param attr: attributes keywords, ascii or unicode
+        """
         self.tag_stack.append((tag, attrs))
         node = self.document.createElement(tag)
         for name, value in attrs.items():
             if value:
-                node.setAttribute(name, str(value))
+                node.setAttribute(name, unicode(value))
         self.position.appendChild(node)
         self.position = node
         return ''
@@ -88,13 +94,17 @@ class Formatter(FormatterBase):
         """ low level function: closes tag right now
             must be the last opend tag!!!
         """
+        if tag=='p': self.in_p = 0 #XXX
         if self.tag_stack[-1][0] != tag:
             raise ValueError, "<%s> expected <%s> given" % (self.tag_stack[-1][0], tag)
         self.position = self.position.parentNode
         return self.tag_stack.pop()
 
     def _add_tag(self, tag, **attrs):
-        """ low level function: insert self closing tag right now """
+        """ low level function: insert self closing tag right now
+            does check_p
+        """
+        self._check_p(tag)
         node = self.document.createElement(tag)
         for name, value in attrs.items():
             if value:
@@ -102,9 +112,9 @@ class Formatter(FormatterBase):
         self.position.appendChild(node)                        
         return ''
 
-    def text(self, text):
-        self._check_p()
+    def _text(self, text):
         if text.strip():
+            self._check_p()
             self.position.appendChild(self.document.createTextNode(text))
         return ''
 
@@ -123,7 +133,7 @@ class Formatter(FormatterBase):
             # XXX check if enclosing tag is ok
 
             if tag in self.need_p:
-                self._check_p()
+                self._check_p(tag)
 
             self._open_tag(tag, **attrs)
             tags_to_reopen.reverse()
@@ -143,25 +153,32 @@ class Formatter(FormatterBase):
                 elif last_tag in self.format_tags:
                     tags_to_reopen.append(self._close_tag(last_tag))
                 else:
+                    print self.tag_stack
+                    self.request.write(self.document.documentElement.toprettyxml(" "))
                     raise ValueError, "<%s> expected <%s> given" % (last_tag, tag)
             self._close_tag(tag)
             tags_to_reopen.reverse()
             for tag_name, args in tags_to_reopen:
-                self._open_tag(tag_name, args)
+                self._open_tag(tag_name, **args)
         return ''
 
-    def _check_p(self):
+    def _check_p(self, opening_tag=None):
+        if ((opening_tag is not None) and
+            (opening_tag not in self.need_p)):
+            return
         for tag in self.tag_stack:
             if tag[0] in self.no_p_after: return
-        self._open_tag('p')
+        if self.in_p: return
+        self.in_p = 1
+        self._open_tag('p', type=str(opening_tag))
 
-    def sysmsg(self, text, **kw):
+    def sysmsg(self, on, **kw):
         """ Emit a system message (embed it into the page).
 
             Normally used to indicate disabled options, or invalid
             markup.
         """
-        return text
+        return self._set_tag('sysmesg', on, **kw)
 
     def startDocument(self, pagename):
         return ""
@@ -170,6 +187,9 @@ class Formatter(FormatterBase):
         #return self.document.documentElement.toxml()
         return self.document.documentElement.toprettyxml("  ")
 
+    def lang(self, on, lang_name):
+        return self._set_tag('lang', on, value=lang_name)
+
     def rawHTML(self, markup):
         """ This allows emitting pre-formatted HTML markup, and should be
             used wisely (i.e. very seldom).
@@ -182,28 +202,31 @@ class Formatter(FormatterBase):
         self.text(markup)
         return ''
 
-    def pagelink(self, pagename, text=None, **kw):
-        apply(FormatterBase.pagelink, (self, pagename, text), kw)
-        node = self.document.createElement('pagelink')
-        node.setAttribute('pagename', pagename)
-        
-        if text:
-            node.appendChild(self.document.createTextNode(text))
-        self.position.appendChild(node)                                
-        return ''
+    def pagelink(self, on, pagename='', page=None, **kw):
+        apply(FormatterBase.pagelink, (self, pagename, page), kw)
+        if not pagename and page is not None:
+            pagename = page.page_name
+        kw['pagename'] = pagename
+        return self._set_tag('pagelink', on,  **kw)
+
+    def interwikilink(self, on, interwiki='', pagename='', **kw):
+        kw['wiki'] = interwiki
+        kw['pagename'] = pagename
+        return self._set_tag('interwiki', on, **kw)
     
     def macro(self, macro_obj, name, args):
         # call the macro
         return self._add_tag('macro', name=name, args=(args or ''))
 
-    def processor(self, processor_name, lines):
+    def processor(self, processor_name, lines, is_parser = 0):
         """ processor_name MUST be valid!
             writes out the result insted of returning it!
         """
         node = self.document.createElement('processor')
         node.setAttribute('name', processor_name)
+        node.setAttribute('isparser', is_parser)
         node.appendChild(self.document.createTextNode('\n'.join(lines)))
-        return (self._set_tag('processor', True, name=processor_name) +
+        return (self._set_tag('processor', True, name=processor_name, isparser=is_parser) +
                 self.text('\n'.join(lines)) +
                 self._set_tag('processor', False))
 
@@ -215,17 +238,36 @@ class Formatter(FormatterBase):
         else:
             return ''
 
-    def url(self, url, text=None, css=None, **kw):
-        node = self.document.createElement('a')
-        node.setAttribute('url', url)
-        #XXX other parameters
-        if text:
-            node.appendChild(self.document.createTextNode(text))
-        self.position.appendChild(node)
-        return ''
+    def url(self, on, url='', css=None, **kw):
+        kw['href'] = str(url)
+        if css:
+            kw['class'] = str(css)
+        return self._set_tag('a', on,  **kw)
+
+    def attachment_link(self, on, url='', **kw):
+        kw['href'] = url
+        return self._set_tag('attachment', on, **kw)
+    
+    def attachment_image(self, url, **kw):
+        kw['href'] = url
+        kw['type'] = 'image'
+        return self._add_tag('attachment', **kw)
+    
+    def attachment_drawing(self, url, **kw):
+        kw['href'] = url
+        kw['type'] = 'drawing'
+        return self._add_tag('attachment', **kw)
+    
+    def attachment_inlined(self, url, **kw):
+        kw['href'] = url
+        kw['type'] = 'inline'
+        return self._add_tag('attachment', **kw)
 
     def rule(self, size=0):
-        return self._add_tag('hr', {'size': str(size)})
+        return self._add_tag('hr', size=str(size))
+
+    def icon(self, type):
+        return self._add_tag('icon', type=type)
 
     def strong(self, on):
         return self._set_tag('b', on)
@@ -233,9 +275,8 @@ class Formatter(FormatterBase):
     def emphasis(self, on):
         return self._set_tag('em', on)
 
-    #def highlight(self, on):
-    #    self._set_tag('highlight', on)
-    #    return ''
+    def highlight(self, on):
+        return self._set_tag('highlight', on)
 
     def number_list(self, on, type=None, start=None):
         return self._set_tag('ol', on, type=type, start=start)
@@ -260,7 +301,7 @@ class Formatter(FormatterBase):
         return self._set_tag('pre', on)
 
     def paragraph(self, on):
-        self.in_p = on != 0
+        FormatterBase.paragraph(self, on)
         return self._set_tag('p', on)
 
     def linebreak(self, preformatted=1):
@@ -269,30 +310,32 @@ class Formatter(FormatterBase):
         else:
             return self._add_tag('br')
                                   
-    def heading(self, depth, title, **kw):
-        if title == None: title = ''
-        if kw.has_key('on'):
-            return self._set_tag('h%d' %depth, kw['on'])
-        
-        return (self._set_tag('h%d' % depth, True, **kw) +
-                self.text(title) +
-                self._set_tag('h%d' % depth, False))                                  
+    def heading(self, on, depth, **kw):
+        return self._set_tag('h%d' %depth, on, **kw)
+
+    def _check_attrs(self, attrs):
+        result = {}
+        for name, value in attrs.iteritems():
+            result[str(name)] = value
+        return result
 
     def table(self, on, attrs={}):
-        return self._set_tag('table', on, attrs)
+        return self._set_tag('table', on, **self._check_attrs(attrs))
         
     def table_row(self, on, attrs={}):
-        return self._set_tag('tr', on, attrs)
+        return self._set_tag('tr', on, **self._check_attrs(attrs))
 
 
     def table_cell(self, on, attrs={}):
-        return self._set_tag('td', on, attrs)
+        return self._set_tag('td', on, **self._check_attrs(attrs))
 
     def anchordef(self, name):
         return self._add_tag('anchor', name=name)
 
-    def anchorlink(self, name, text): # XXX TODO add missing id keyword parameter
-        return self.url("#" + name, text)
+    def anchorlink(self, on, name, id = None):
+        kw = {}
+        if id: kw['id'] = str(id)
+        return self.url(on, "#" + name, **kw)
 
     def underline(self, on):
         return self._set_tag('u', on)
@@ -301,7 +344,7 @@ class Formatter(FormatterBase):
         return self._set_tag('dl', on)
 
     def definition_term(self, on, compact=0):
-        # XXXX may be not correct
+        # XXX may be not correct
         # self._langAttr() missing
         if compact and on:
             return self._set_tag('dt compact', on)
@@ -310,7 +353,7 @@ class Formatter(FormatterBase):
 
     def definition_desc(self, on):
         # self._langAttr() missing
-        self._set_tag('dd', on)
+        return self._set_tag('dd', on)
 
     def image(self, **kw):
         """ Take HTML <IMG> tag attributes in `attr`.
@@ -319,3 +362,26 @@ class Formatter(FormatterBase):
         """
         return self._add_tag('img', **kw)
 
+    def escapedText(self, text):
+        return wikiutil.escape(text)
+
+    def small(self, on):
+        return self._set_tag('small', on)
+
+    def big(self, on):
+        return self._set_tag('big', on)
+
+    def code_area(self, on, code_id, code_type='code', show=0, start=-1, step=-1):
+        kw = {'id' : code_id,
+              'type' : code_type,
+              'show' : show,
+              }
+        if start != -1: kw['start'] = start
+        if step != -1: kw['step'] = step
+        return self._set_tag('codearea', on, **kw)
+
+    def code_line(self, on):
+        return self._set_tag('codeline', on)
+
+    def code_token(self, on, tok_type):
+        return self._set_tag('codetoken', on, type=tok_type)
