@@ -1,35 +1,28 @@
 # -*- coding: iso-8859-1 -*-
 """
-    MoinMoin - Helper functions for email stuff
+    MoinMoin - email helper functions
 
     @copyright: 2003 by Jrgen Hermann <jh@web.de>
     @license: GNU GPL, see COPYING for details.
 """
 
-# Imports
 import os
 
 _transdict = {"AT": "@", "DOT": ".", "DASH": "-"}
 
 
-#############################################################################
-### Mail
-#############################################################################
-
 def sendmail(request, to, subject, text, **kw):
-    """
-    Send a mail to the address(es) in 'to', with the given subject and
-    mail body 'text'.
-    
+    """ Create and send a text/plain message
+        
     Return a tuple of success or error indicator and message.
-
-    Set a different "From" address with "mail_from=<email>".
+    
     @param request: the request object
-    @param to: target email address
-    @param subject: subject of email
-    @param text: email body text
+    @param to: recipients (list)
+    @param subject: subject of email (unicode)
+    @param text: email body text (unicode)
+    @keyword mail_from: override default mail_from (string)
     @rtype: tuple
-    @return: (is_ok, msg)
+    @return: (is_ok, Description of error or OK message)
     """
     import smtplib, socket
     from email.MIMEText import MIMEText
@@ -38,33 +31,48 @@ def sendmail(request, to, subject, text, **kw):
     from MoinMoin import config
 
     _ = request.getText
-    # should not happen, but who knows ...
-    if not config.mail_smarthost:
-        return (0, _('''This wiki is not enabled for mail processing. '''
-                '''Contact the owner of the wiki, who can either enable email, or remove the "Subscribe" icon.'''))
-    mail_from = kw.get('mail_from', config.mail_from) or config.mail_from
+    cfg = request.cfg    
+    mail_from = kw.get('mail_from', '') or cfg.mail_from
 
-    # Create a text/plain message
+    # Create a text/plain message body (see RFC2822)
+    # Replace LF with CRLF, encode using config.charset.
+    text = text.replace(u'\n', u'\r\n')
+    text = text.encode(config.charset)
     msg = MIMEText(text, 'plain', config.charset)
-    msg['From'] = mail_from
-    msg['To'] = mail_from # do not disclose ANY users addr
-    msg['Date'] = formatdate()
     
-    try: # only python >= 2.2.2 has this:
+    # Create message headers
+    msg['From'] = mail_from
+    # Don't expose emails addreses of the other subscribers, instead we
+    # use the same mail_from, e.g. "My Wiki <noreply@mywiki.org>"
+    msg['To'] = mail_from
+    msg['Date'] = formatdate()
+
+    try:
+        # Python 2.2.2 and later has a message id function and an Header
+        # class that allow us to use any charset in the email header.
         from email.Header import Header
         from email.Utils import make_msgid
-        msg['Message-ID'] = make_msgid() 
+        msg['Message-ID'] = make_msgid()
         msg['Subject'] = Header(subject, config.charset)
     except ImportError:
-        msg['Subject'] = subject # this is not standards compliant, but mostly works
-        # no message-id. if you still have py 2.2.1, you like it old and broken
-        
+        # In older version you will get no message id and broken non
+        # standard compliant headers, but it mostly works.
+        msg['Subject'] = subject
+
+    # Send the message
     try:
-        server = smtplib.SMTP(config.mail_smarthost)
+        server = smtplib.SMTP(cfg.mail_smarthost)
         try:
             #server.set_debuglevel(1)
-            if config.mail_login:
-                user, pwd = config.mail_login.split()
+            if cfg.mail_login:
+                user, pwd = cfg.mail_login.split()
+                try: # try to do tls
+                    server.ehlo()
+                    if server.has_extn('starttls'):
+                        server.starttls()
+                        server.ehlo()
+                except:
+                    pass
                 server.login(user, pwd)
             server.sendmail(mail_from, to, msg.as_string())
         finally:
@@ -77,7 +85,7 @@ def sendmail(request, to, subject, text, **kw):
         return (0, str(e))
     except (os.error, socket.error), e:
         return (0, _("Connection to mailserver '%(server)s' failed: %(reason)s") % {
-            'server': config.mail_smarthost, 
+            'server': cfg.mail_smarthost, 
             'reason': str(e)
         })
 
@@ -85,8 +93,10 @@ def sendmail(request, to, subject, text, **kw):
 
 
 def decodeSpamSafeEmail(address):
-    """
-    Decode a spam-safe email address in `address` by applying the following rules.
+    """ Decode obfuscated email address to stanard email address
+
+    Decode a spam-safe email address in `address` by applying the
+    following rules:
     
     Known all-uppercase words and their translation:
         "DOT"   -> "."
